#!/usr/bin/python
#
# Python control of stepper motors.
#
# Copyright (C) 2008-2009, W. Trevor King
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
#  * Neither the name of the copyright holders nor the names of the
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from pycomedi.single_dio import DO_port
write = DO_port(chan=(0,1,2,3))

from time import sleep # in case our DIO is too fast for the motor to keep up

VERSION = "0.2"

port_vals = [1,  # binary ---1
             5,  # binary -1-1 
             4,  # binary -1--
             6,  # binary -11-
             2,  # binary --1-
             10, # binary 1-1-
             8,  # binary 1---
             9]  # binary 1--1

class stepper_obj :
    def __init__(self) :
        self._curPos = 0 # Current stepper index
        self.fullStep = True # Select full or half stepping
        self.logic = False # Select active high (True) or active low (False)
        self.delay = 1e-5 # 10 us
        self._set_pos(0)
        self.setExternalStepPos = None
    def __del__(self) :
        pass
    def _get_output(self, position) :
        return port_vals[position % 8]
    def _set_pos(self, position) :
        write(self._get_output(position))
    def get_cur_pos(self) :
        return self._curPos
    def single_step(self, direction) :
        if self.fullStep and self._curPos % 2 == 1 :
            self._curPos -= 1 # round down to a full step
        if direction > 0 :
            if self.fullStep :
                self._curPos += 2
            else :
                self._curPos += 1
        if direction < 0 :
            if self.fullStep :
                self._curPos -= 2
            else :
                self._curPos -= 1
        write(self._get_output(self._curPos))
        if self.setExternalStepPos != None :
            self.setExternalStepPos(self.get_cur_pos())
        if self.delay > 0 :
            sleep(self.delay)
    def step_to(self, target_position) :
        if target_position != int(target_position) :
            str = "target_position must be an int: ", target_position
            raise Exception, str
        if self.fullStep and target_position%2 == 1 :
            target_position -= 1
        if target_position > self._curPos :
            direction = 1
        else :
            direction = -1
        while self._curPos != target_position :
            self.single_step(direction)
    def step_rel(self, relative_target_position) :
        return self.step_to(self._curPos + relative_target_position)

def test() :
    s = stepper_obj()
    s._get_output(0)
    s._get_output(-100)
    s._get_output(100)
    s.fullStep = True
    s.single_step(1)
    s.single_step(1)
    s.single_step(1)
    s.single_step(-1)
    s.single_step(-1)
    s.single_step(-1)
    s.fullStep = False
    s.single_step(-1)
    s.single_step(-1)
    s.single_step(-1)
    s.single_step(1)
    s.single_step(1)
    s.single_step(1)
    s.step_to(1000)
    s.step_to(-1000)
    s.step_to(0)

if __name__ == "__main__" :
    test()
